@@ -1,9 +1,10 @@
 from collections.abc import Iterable, Callable
 from functools import wraps
-from typing import Literal, Self, Union, Any
+from typing import Literal, Union, Any
+from SwiftGUI.Compat import Self
 import tkinter as tk
 
-from SwiftGUI import Event, GlobalOptions, Color
+from SwiftGUI import Event, GlobalOptions, Color, remove_None_vals, Literals
 from SwiftGUI.ElementFlags import ElementFlag
 #from SwiftGUI.Widget_Elements.Frame import Frame
 
@@ -53,7 +54,7 @@ class BaseElement:
     _element_flags:set[ElementFlag] # Properties the element has
     window = None # Main Window
 
-    key:any = None  # If given, this will be written to the event-value. Elements without a key can not throw key-events
+    key:Any = None  # If given, this will be written to the event-value. Elements without a key can not throw key-events
     _key_function: Callable | Iterable[Callable] = None  # Called as an event
 
     defaults:type[GlobalOptions.DEFAULT_OPTIONS_CLASS] = GlobalOptions.Common  # Change this to apply a different default configuration
@@ -65,6 +66,8 @@ class BaseElement:
     def __init__(self):
         self._run_when_window_exists = list()
 
+        self._update_options = dict()   # This saves all valuew passed by .update, so they can be recalled using .get_option()
+
     def _init(self,parent:"BaseElement",window):
         """
         Not __init__
@@ -154,7 +157,7 @@ class BaseElement:
         """
         ...
 
-    def _get_value(self) -> any:
+    def _get_value(self) -> Any:
         """
         Returns the value(s) of the Element.
         Override this function.
@@ -162,16 +165,21 @@ class BaseElement:
         """
         return None
 
-    def set_value(self,val:any):
+    def set_value(self,val:Any) -> Self:
         """
         Set the value of the element
         :param val: New value
         :return:
         """
-        pass
+        return self
+
+    def __call__(self, val: Any) -> Self:
+        """Same as .set_value"""
+        self.set_value(val)
+        return self
 
     @property
-    def value(self) -> any:
+    def value(self) -> Any:
         """
         Value of the surrounding object.
         Override _get_value to create custom values
@@ -201,7 +209,7 @@ class BaseElement:
         """
         pass
 
-    def _update_special_key(self,key:str,new_val:any) -> bool|None:
+    def _update_special_key(self,key:str,new_val:Any) -> bool|None:
         """
         Inherit this method to pick out "special" keys to update.
         Keys are passed one-by-one.
@@ -229,7 +237,7 @@ class BaseElement:
         """
         ...
 
-    def update(self,**kwargs):
+    def _update_initial(self,**kwargs) -> Self:
         """
         Update configurations of this element.
         :param kwargs:
@@ -237,12 +245,52 @@ class BaseElement:
         """
 
         kwargs = self.defaults.apply(kwargs)
+
+        self._update_options = kwargs.copy()
+
         kwargs = dict(filter(lambda a: not self._update_special_key(*a), kwargs.items()))
         self._update_default_keys(kwargs)
 
         if self.has_flag(ElementFlag.IS_CREATED) and self.window.has_flag(ElementFlag.IS_CREATED):
             self._apply_update()
 
+        return self
+
+    def update(self, **kwargs) -> Self:
+        """
+        Standard update-routine.
+        None-values are removed.
+        :param kwargs:
+        :return:
+        """
+        kwargs = remove_None_vals(kwargs)
+
+        self._update_options.update(kwargs)
+
+        self._update_initial(**kwargs)
+
+        return self
+
+    def get_option(self, key: str, default: Any = None) -> Any:
+        """
+        Returns the option like it was set in .update.
+        If it wasn't set in .update, it tries to get the option from the widget itself (only for BaseWidget)
+
+        :param key:
+        :param default:
+        :return:
+        """
+        return self._update_options.get(key, default)
+
+    def update_to_default_value(self, *options: str) -> Self:
+        """
+        Apply the global options to certain keys, aka reset that key to its default value
+        :param options:
+        :return:
+        """
+        self._update_initial(**{key: None for key in options})
+        return self
+
     @run_after_window_creation
     def update_after_window_creation(self, **kwargs) -> Self:
         """
@@ -250,7 +298,7 @@ class BaseElement:
         :param kwargs:
         :return:
         """
-        self.update(**kwargs)
+        self._update_initial(**kwargs)
         return self
 
     # @run_after_window_creation
@@ -268,12 +316,15 @@ class BaseElement:
         ONLY FOR INHERITING CLASSES!!!
         DON'T CALL!!!
 
-        Will be called oncce as soon as the element exists
+        Will be called once all elements exist
         :return:
         """
         for fkt in self._run_when_window_exists:    # Call all those buffered functions
             fkt()
 
+    def __repr__(self) -> str:
+        return f"<sg.{self.__class__.__name__} element at {id(self)}>"
+
 
 class BaseWidget(BaseElement):
     """
@@ -284,6 +335,8 @@ class BaseWidget(BaseElement):
     _tk_widget_class:type = None # Class of the connected widget
     _tk_kwargs:dict = dict()
 
+    _grab_anywhere_on_this: bool = False    # If True, you can grab windows by grabbing this element
+
     # _transfer_keys = {  # Usual couple of keys
     #     "background_color_disabled":"disabledbackground",
     #     "background_color":"background",
@@ -308,7 +361,9 @@ class BaseWidget(BaseElement):
 
     _events_to_bind_later: list[dict]
 
-    def __init__(self,key:any=None,tk_kwargs:dict[str:any]=None,expand:bool = False,expand_y:bool = False,**kwargs):
+    _tk_scrollbar_y: tk.Scrollbar | None = None
+
+    def __init__(self,key:Any=None,tk_kwargs:dict[str:Any]=None,expand:bool = False,expand_y:bool = False,**kwargs):
         super().__init__()
         self._events_to_bind_later = list()
 
@@ -343,7 +398,7 @@ class BaseWidget(BaseElement):
     def tk_widget(self) -> tk.Widget:
         return self._tk_widget
 
-    def bind_event(self,tk_event:str|Event,key_extention:Union[str,any]=None,key:any=None,key_function:Callable|Iterable[Callable]=None)->Self:
+    def bind_event(self,tk_event:str|Event,key_extention:Union[str,Any]=None,key:Any=None,key_function:Callable|Iterable[Callable]=None)->Self:
         """
         Bind a tk-event onto the underlying tk-widget
 
@@ -393,7 +448,7 @@ class BaseWidget(BaseElement):
         return self
 
     def _init_defaults(self):
-        self.update(**self._tk_kwargs)
+        self._update_initial(**self._tk_kwargs)
 
     def _init_widget_for_inherrit(self,container) -> tk.Widget:
         """
@@ -424,7 +479,7 @@ class BaseWidget(BaseElement):
         self._tk_target_value = variable
         self._tk_kwargs[kwargs_key] = variable
 
-    def _set_tk_target_variable(self,value_type:type=tk.StringVar,kwargs_key:str="textvariable",default_key:str=None,default_value:any=None):
+    def _set_tk_target_variable(self,value_type:type=tk.StringVar,kwargs_key:str="textvariable",default_key:str=None,default_value:Any=None):
         """
         Define a target variable for this widget
         :param default_key: If given and in self._tk_args, self._tk_args[default_key] will be the default value. The key will be removed from the dict.
@@ -440,27 +495,35 @@ class BaseWidget(BaseElement):
         self._tk_target_value = value_type(self.parent_tk_widget, value=default_value)
         self._tk_kwargs[kwargs_key] = self._tk_target_value
 
-    def _init_widget(self,container:tk.Widget|tk.Tk,mode:Literal["pack","grid"]="pack") -> None:
+    def _init_widget(self,container:tk.Widget|tk.Tk) -> None:
         """
         Initialize the widget to the container
         :return:
         """
         self._tk_widget = self._init_widget_for_inherrit(container)
 
-        match mode:
-            case "pack":
-                #temp = {"expand":False,"side":"left"}
-                temp = {"side":"left"}
+        #temp = {"expand":False,"side":"left"}
+        temp = {"side":"left"}
 
-                temp.update(self._insert_kwargs)
+        temp.update(self._insert_kwargs)
 
-                if self.has_flag(ElementFlag.EXPAND_ROW) or self.has_flag(ElementFlag.EXPAND_VERTICALLY):
-                    temp["expand"] = temp.get("expand",True)
+        if self.has_flag(ElementFlag.EXPAND_ROW) or self.has_flag(ElementFlag.EXPAND_VERTICALLY):
+            temp["expand"] = temp.get("expand",True)
+
+            match (self.has_flag(ElementFlag.EXPAND_ROW), self.has_flag(ElementFlag.EXPAND_VERTICALLY)):
+                case (True, True):
                     temp["fill"] = temp.get("fill","both")
+                case (True, False):
+                    temp["fill"] = temp.get("fill", "x")
+                case (False, True):
+                    temp["fill"] = temp.get("fill", "y")
 
-                self._tk_widget.pack(**temp)
-            case "grid":
-                self._tk_widget.grid(**self._insert_kwargs)
+        self._tk_widget.pack(**temp)
+
+        if hasattr(self, "scrollbar_y"):
+            self.scrollbar_y._init(self.parent, self.window)
+            self.scrollbar_y.bind_to_element(self)
+            self.scrollbar_y.tk_widget.pack(expand= True, fill= "y", side= "left")
 
         if self.has_flag(ElementFlag.IS_CONTAINER):
             self._init_containing()
@@ -519,7 +582,11 @@ class BaseWidget(BaseElement):
                 line.pack(fill="x")
             actual_line.pack(**ins_kwargs_rows)
 
-    def _get_value(self) -> any:
+            if self._grab_anywhere_on_this:
+                self.window.bind_grab_anywhere_to_element(line)
+                self.window.bind_grab_anywhere_to_element(actual_line)
+
+    def _get_value(self) -> Any:
         """
         This method is used when the value/state of the Widget is read.
         :return:
@@ -558,12 +625,14 @@ class BaseWidget(BaseElement):
     def _apply_update(self):
         self._tk_widget.configure(self._tk_kwargs)
 
-    def set_value(self,val:any):
+    def set_value(self,val:Any) -> Self:
         try:
             self._tk_target_value.set(val)
         except AttributeError:
             pass
 
+        return self
+
     def init_window_creation_done(self):
         super().init_window_creation_done()
 
@@ -571,11 +640,68 @@ class BaseWidget(BaseElement):
             self.bind_event(**params)
         del self._events_to_bind_later  # Free some ram, because why not
 
+        if self._grab_anywhere_on_this:
+            self.window.bind_grab_anywhere_to_element(self.tk_widget)
+
+    def get_option(self, key: str, default: Any = None) -> Any:
+        highlevel = super().get_option(key) # Option set by .update ?
+        if highlevel is not None:
+            return highlevel
+
+        try:    # Option in tkinter widget?
+            return self.tk_widget.cget(key)
+        except tk.TclError:
+            pass
+
+        if key in self._transfer_keys:  # Maybe the key is called differently in tkinter?
+            key = self._transfer_keys[key]
+
+            try:
+                return self.tk_widget.cget(key)
+            except tk.TclError:
+                pass
+
+        return default  # Well, gave it my best shot...
+
+class BaseScrollbar:
+    """
+    Base for every widget that has a scrollbar.
+
+    Probably the most annoying feature to implement, but it works now...
+    Appreciate it.
+    """
+    scrollbar_y: BaseWidget # Actually type sg.Scrollbar, but yeah...
+
+    @run_after_window_creation
+    def update_scrollbar_y(
+            self,
+            cursor: Literals.cursor = None,
+            # takefocus: bool = None,
+
+            background_color: str | Color = None,
+            background_color_active: str | Color = None,
+
+            text_color: str | Color = None,
+            text_color_active: str | Color = None,
+
+            troughcolor: str | Color = None,
+    ) -> Self:
+        assert hasattr(self, "scrollbar_y"), f"{self} has no scrollbar, so .update_scrollbar_y can't be called..."
+        self.scrollbar_y.update(
+            cursor = cursor,
+            background_color = background_color,
+            background_color_active = background_color_active,
+            text_color = text_color,
+            text_color_active = text_color_active,
+            troughcolor = troughcolor,
+        )
+        return self
+
 class BaseWidgetContainer(BaseWidget):
     """
     Base for Widgets that contain other widgets
     """
-    def __init__(self,key:any=None,tk_kwargs:dict[str:any]=None,expand:bool = False,expand_y:bool = False,**kwargs):
+    def __init__(self,key:Any=None,tk_kwargs:dict[str:Any]=None,expand:bool = False,expand_y:bool = False,**kwargs):
         super().__init__(key,tk_kwargs,expand,expand_y=expand_y,**kwargs)
 
         self._containing_row_frame_widgets = list()
@@ -586,6 +712,15 @@ class BaseWidgetContainer(BaseWidget):
         self._line_insert_kwargs = dict()
         self.add_flags(ElementFlag.IS_CONTAINER)
 
+    def update_all_containing(self, **kwargs) -> Self:
+        """
+        Update all elements inside this frame
+        :param kwargs:
+        :return:
+        """
+        raise NotImplementedError("update_all_containing isn't implemented yet...")
+        # Todo: update_all_containing
+
 class BaseWidgetTTK(BaseWidget):
     _styletype: str = None  # Style will be named n.styletype
     _style: str = None  # Registered style of this widget
@@ -667,70 +802,3 @@ class BaseWidgetTTK(BaseWidget):
 
 
         self.window.ttk_style.map(stylename, **new_kwargs)
-
-class BaseCombinedElement(BaseElement):
-    """
-    Derive from this class to create an element consisting of multiple inner elements.
-    """
-    def __init__(
-            self,
-            frame, key: Any = None,
-            key_function: Callable | Iterable[Callable] = None,
-            apply_parent_background_color: bool = True
-    ):
-        """
-
-        :param frame: Pass a Frame containing all the elements you'd like to have inside this element
-        :param key: Pass a key to register it in main window
-        :param apply_parent_background_color: True, if the background_color of the parent container should also apply to this frame
-        """
-        super().__init__()
-
-        self._sg_widget = frame
-        self.key = key
-        self._key_function = key_function
-
-        self._throw_event: Callable = lambda :None
-
-        if apply_parent_background_color:
-            self.add_flags(ElementFlag.APPLY_PARENT_BACKGROUND_COLOR)
-
-    def throw_event(self):
-        """
-        Throw the default event to window
-        :return:
-        """
-        self._throw_event()
-
-    def _personal_init(self):
-        self._sg_widget._init(self,self.window)
-        self._throw_event = self.window.get_event_function(me= self, key= self.key, key_function= self._key_function)
-
-    def _update_special_key(self,key:str,new_val:any) -> bool|None:
-        """
-        Inherit (use) this method to pick out "special" keys to update.
-        Keys are passed one-by-one.
-
-        When calling .update, this method gets called first.
-        If it returns anything truethy, execution of .update ends for this key.
-
-        Otherwise, ._update_default_keys gets called for the key.
-
-        Just copy the whole method and add more cases.
-
-        :param key:
-        :param new_val:
-        :return:
-        """
-        match key:
-            case "background_color":
-                self._sg_widget.update(background_color = new_val)
-            case _:
-                # The key wasn't found in any other case
-                return super()._update_special_key(key, new_val)    # Look in the parent-class
-
-        # The key was found in match-case
-        return True
-
-    # def _throw_event(self):
-    #     self.window.
