@@ -2,9 +2,10 @@
 #from tkinter import ttk
 from collections.abc import Iterable
 from os import PathLike
-from typing import Literal, Union
+from typing import Literal, Union, Any, Callable
 
-from SwiftGUI import Literals, Color, font_windows, Font
+from SwiftGUI import Literals, Color, font_windows, Font, Extras
+from SwiftGUI.Utilities.Images import file_from_b64
 
 # Every option-class will be stored in here
 all_option_classes:list[Union["_DefaultOptionsMeta",type]] = list()
@@ -125,7 +126,7 @@ class DEFAULT_OPTIONS_CLASS(metaclass=_DefaultOptionsMeta):
         return apply_to
 
     @classmethod
-    def single(cls,key:str,val:any,default:any=None) -> any:
+    def single(cls,key:str,val:Any = None,default:Any=None) -> Any:
         """
         val will be returned.
         If val is None, cls.key will be returned.
@@ -152,6 +153,7 @@ class Common(DEFAULT_OPTIONS_CLASS):
     takefocus:bool = True
     expand:bool = False
     expand_y: bool = False
+    highlightcolor: Color | str = None
 
 class Common_Background(DEFAULT_OPTIONS_CLASS):
     """
@@ -189,6 +191,35 @@ class Text(Common, Common_Textual, Common_Background):
     padding:Literals.padding = 0
     width:int = None
 
+class Scale(Common_Background, Common_Textual):
+    default_value: int | float = None
+    number_min: float = None
+    number_max: float = None
+    resolution: float = None
+    showvalue: bool = None
+    tickinterval: float = None
+    width: int = None
+    length: int = None
+    sliderlength: int = None
+    sliderrelief: Literals.relief = None
+    orient: Literal["horizontal", "vertical"] = "horizontal"
+    disabled: bool = None
+    readonly: bool = None
+    borderwidth: int = None
+    label: str = None
+    troughcolor: str | Color = None
+    digits: int = None
+    cursor: Literals.cursor = None
+    takefocus: bool = None
+    apply_parent_background_color: bool = True
+    relief: Literals.relief = None
+    highlightbackground_color: str | Color = None
+    highlightcolor: str | Color = None
+    highlightthickness: int = 0
+    repeatdelay: int = None
+    repeatinterval: int = None
+    background_color_active: str | Color = None
+
 class Input(Common,Common_Textual,Common_Field_Background):
     text: str = None
     width: int = None
@@ -205,7 +236,6 @@ class Input(Common,Common_Textual,Common_Field_Background):
     selectbackground_color: str | Color = None
     select_text_color: str | Color = None
     selectborderwidth: int = None
-    highlightcolor: str | Color = None
     highlightthickness: int = None
     pass_char: str = None
     disabled: bool = None  # Set state to tk.Normal, or 'disabled'
@@ -213,6 +243,7 @@ class Input(Common,Common_Textual,Common_Field_Background):
     exportselection: bool = None
     validate: Literals.validate = None
     validatecommand: callable = None
+    cursor_color: str | Color = None
     #
     # Mixed options
 
@@ -242,6 +273,9 @@ class Button(Common,Common_Textual,Common_Field_Background):
     repeatdelay: int = None
     repeatinterval: int = None
 
+class MultistateButton(Button):
+    can_deselect: bool = True
+
 class Frame(Common, Common_Background):
     takefocus = False
     padding: Literals.padding = 3
@@ -252,12 +286,13 @@ class Frame(Common, Common_Background):
 
     borderwidth: int = None
     highlightbackground_color: Color | str = None
-    highlightcolor: Color | str = None
     highlightthickness: int = None
 
     padx: int = 2
     pady: int = 2
 
+class GridFrame(Frame):
+    ...
 
 class Checkbox(Common,Common_Textual, Common_Background):
     default_value: bool = False
@@ -285,33 +320,34 @@ class Checkbox(Common,Common_Textual, Common_Background):
     offrelief: Literals.relief = None
     relief: Literals.relief = None
     # hilightbackground_color: str | Color = None
-    # highlightcolor: str | Color = None
 
 class Radiobutton(Checkbox):
     # hilightbackground_color: str | Color = None,
-    # highlightcolor: str | Color = None,
     # highlightthickness: int = None,
     ...
 
 class Window(Common_Background):
     title = "SwiftGUI Window"
     titlebar: bool = True  # Titlebar visible
-    resizeable_width = False
-    resizeable_height = False
+    resizeable_width: bool = False
+    resizeable_height: bool = False
     fullscreen: bool = False
     transparency: Literals.transparency = 0  # 0-1, 1 meaning invisible
-    size: tuple[int, int] = (None, None)
+    size: int | tuple[int, int] = (None, None)
     position: tuple[int, int] = (None, None)  # Position on monitor # Todo: Center
-    min_size: tuple[int, int] = (None, None)
-    max_size: tuple[int, int] = (None, None)
-    icon: str = None  # .ico file
+    min_size: int | tuple[int, int] = (None, None)
+    max_size: int | tuple[int, int] = (None, None)
+    icon: str = file_from_b64(Extras.SwiftGUI.icon)
     keep_on_top: bool = False
     ttk_theme: str = "default"
+    grab_anywhere: bool = False
 
 class Listbox(Common,Common_Textual,Common_Field_Background):
+    no_selection_returns: Any = ""  # Returned when nothing is selected
     activestyle:Literals.activestyle = "none"
     default_list: Iterable[str] = None
     disabled: bool = None
+    scrollbar: bool = True
     borderwidth: int = None
     background_color_selected: str | Color = None
     selectborderwidth: int = None
@@ -322,9 +358,11 @@ class Listbox(Common,Common_Textual,Common_Field_Background):
     height: int = None
     relief: Literals.relief = None
     highlightbackground_color: str | Color = None
-    highlightcolor: str | Color = None
     highlightthickness: int = None
 
+class Scrollbar(Scale):
+    ...
+
 class FileBrowseButton(Button):
     file_browse_type: Literals.file_browse_types = "open_single"
     file_browse_filetypes: Literals.file_browse_filetypes = (("All files","*"),)
@@ -337,20 +375,12 @@ class FileBrowseButton(Button):
 class ColorChooserButton(Button):
     color_chooser_title: str = None
 
-class TextField(Common,Common_Textual,Common_Field_Background):
+class TextField(Input):
     borderwidth: int = None
-    width: int = None
+    scrollbar: bool = False
     height: int = None
     insertbackground: str | Color = None
-    highlightbackground_color: str | Color = None
-    selectbackground_color: str | Color = None
-    select_text_color: str | Color = None
-    selectborderwidth: int = None
-    highlightcolor: str | Color = None
-    highlightthickness: int = None
     readonly: bool = False  # Set state to tk.Normal, or 'readonly'
-    relief: Literals.relief = None
-    exportselection: bool = False
     padx: int = None
     pady: int = None
 
@@ -377,7 +407,6 @@ class Table(Common, Common_Textual,Common_Field_Background):
     font_underline_headings: bool = None
     font_overstrike_headings: bool = None
 
-    background_color: str | Color = None
     background_color_rows: str | Color = None
     background_color_active_rows: str | Color = Color.light_blue
 
@@ -390,6 +419,7 @@ class Table(Common, Common_Textual,Common_Field_Background):
 
     sort_col_by_click: bool = True
     takefocus:bool = False
+    scrollbar: bool = True
 
     selectmode: Literals.selectmode_tree = "browse"
     cursor: Literals.cursor = None
@@ -400,6 +430,7 @@ class Table(Common, Common_Textual,Common_Field_Background):
 class Separator(Common_Background):
     color: str | Color = Color.light_grey
     weight: int = 2
+    padding: int = 3
 
 class SeparatorHorizontal(Separator):
     ...
@@ -432,6 +463,51 @@ class Notebook(Common_Textual, Common_Background):
 class LabelFrame(Frame, Common_Textual):
     relief: Literals.relief = "solid"
     labelanchor: Literals.tabposition = "nw"
+    no_label: bool = False
+
+class TabFrame(Frame):
+    text: str = None
+
+class Spinbox(Button, Input):
+    default_value: float = None
+    cursor_button: Literals.cursor = None
+    background_color_disabled: str | Color = None
+    background_color_button: Color | str = None
+    relief_button_down: Literals.relief = None
+    relief_button_up: Literals.relief = None
+    values: Iterable[float] = None
+    wrap: bool = None
+    number_format: str = None
+    number_min: float = None
+    number_max: float = None
+    increment: float = None
+    repeatdelay: int = 300
+    repeatinterval: int = 50
+    state: Literals.Spinbox_State = None
+
+class Combobox(Button, Input):
+    background_color_disabled: str | Color = None
+    button_background_color = None
+    button_background_color_active = None
+    arrow_color = None
+    arrow_color_active = None
+    can_change_text: bool = False
+    insertbackground: str | Color = None
+
+class Image(Common_Background):
+    height: int = None
+    width: int = None
+    apply_parent_background_color: bool = True
+
+class Progressbar(Common_Field_Background):
+    number_max: float = None
+    cursor: Literals.cursor = None
+    bar_color: str | Color = None
+    takefocus: bool = None
+    mode: Literals.progress_mode = "determinate"
+
+class ImageButton(Button):
+    ...
 
 def reset_all_options():
     """
