@@ -1,11 +1,15 @@
+import threading
+import time
 import tkinter
 import tkinter.ttk as ttk
 from collections.abc import Iterable, Callable, Iterator
 from functools import partial
 from tkinter import font
-from typing import Any, Self
+from typing import Any, Literal
+from SwiftGUI.Compat import Self
+from itertools import islice
 
-from SwiftGUI import ElementFlag, GlobalOptions, Literals, Color, BaseWidgetTTK, BaseElement
+from SwiftGUI import ElementFlag, GlobalOptions, Literals, Color, BaseWidgetTTK, BaseElement, Scrollbar, BaseScrollbar
 
 
 class TableRow(list):
@@ -110,7 +114,7 @@ class TableRow(list):
 
         return super().__eq__(other)
 
-class Table(BaseWidgetTTK):
+class Table(BaseWidgetTTK, BaseScrollbar):
     tk_widget:ttk.Treeview
     _tk_widget:ttk.Treeview
     _tk_widget_class:type = ttk.Treeview # Class of the connected widget
@@ -139,6 +143,7 @@ class Table(BaseWidgetTTK):
 
             sort_col_by_click: bool = None,
             selectmode: Literals.selectmode_tree = None,
+            scrollbar: bool = None,
 
             # Fonts
             fonttype: str = None,
@@ -173,10 +178,16 @@ class Table(BaseWidgetTTK):
             takefocus: bool = None,
             expand: bool = None,
             expand_y: bool = None,
-            tk_kwargs: dict[str:any]=None
+            tk_kwargs: dict[str:Any]=None
     ):
         super().__init__(key=key,tk_kwargs=tk_kwargs,expand=expand, expand_y = expand_y)
 
+        self._thread_lock = threading.Lock()
+
+        if self.defaults.single("scrollbar", scrollbar):
+            self.add_flags(ElementFlag.HAS_SCROLLBAR_Y)
+            self.scrollbar_y = Scrollbar(expand_y= True)
+
         self._elements = list()
         self._element_dict = dict()
 
@@ -190,48 +201,20 @@ class Table(BaseWidgetTTK):
         if tk_kwargs is None:
             tk_kwargs = dict()
 
-        self.update(
-            columns = self._headings,
-            column_width = column_width,
-            **tk_kwargs,
-            selectmode = selectmode,
-
-            background_color = background_color,
-
-            background_color_rows = background_color_rows,
-            background_color_active_rows = background_color_active_rows,
-
-            background_color_headings = background_color_headings,
-            background_color_active_headings = background_color_active_headings,
-
-            text_color= text_color,
-            text_color_active= text_color_active,
-            text_color_headings= text_color_headings,
-            text_color_active_headings = text_color_active_headings,
-
-            fonttype = fonttype,
-            fontsize = fontsize,
-            font_bold = font_bold,
-            font_italic = font_italic,
-            font_underline = font_underline,
-            font_overstrike = font_overstrike,
-
-            fonttype_headings=fonttype_headings,
-            fontsize_headings=fontsize_headings,
-            font_bold_headings=font_bold_headings,
-            font_italic_headings=font_italic_headings,
-            font_underline_headings=font_underline_headings,
-            font_overstrike_headings=font_overstrike_headings,
-
-            sort_col_by_click=sort_col_by_click,
-            takefocus=takefocus,
-
-            # New ones
-            height = height,
-            cursor = cursor,
-            padding = padding,
-
-        )
+        self._update_initial(columns=self._headings, column_width=column_width, selectmode=selectmode,
+                             background_color=background_color, background_color_rows=background_color_rows,
+                             background_color_active_rows=background_color_active_rows,
+                             background_color_headings=background_color_headings,
+                             background_color_active_headings=background_color_active_headings, text_color=text_color,
+                             text_color_active=text_color_active, text_color_headings=text_color_headings,
+                             text_color_active_headings=text_color_active_headings, fonttype=fonttype,
+                             fontsize=fontsize, font_bold=font_bold, font_italic=font_italic,
+                             font_underline=font_underline, font_overstrike=font_overstrike,
+                             fonttype_headings=fonttype_headings, fontsize_headings=fontsize_headings,
+                             font_bold_headings=font_bold_headings, font_italic_headings=font_italic_headings,
+                             font_underline_headings=font_underline_headings,
+                             font_overstrike_headings=font_overstrike_headings, sort_col_by_click=sort_col_by_click,
+                             takefocus=takefocus, height=height, cursor=cursor, padding=padding, **tk_kwargs)
 
         if default_event:
             self.bind_event("<<TreeviewSelect>>",key=key,key_function=key_function)
@@ -341,7 +324,7 @@ class Table(BaseWidgetTTK):
         raise AttributeError("You tried to set table_elements directly on an sg.Table. If you want to replace the whole table, use .overwrite_table instead.\n"
                              "I highly recommend not overwriting the whole table though, sg.Table offers a lot of methods to change it.")
 
-    def _update_special_key(self,key:str,new_val:any) -> bool|None:
+    def _update_special_key(self,key:str,new_val:Any) -> bool|None:
         match key:
             case "column_width":
                 if new_val is None:
@@ -439,7 +422,7 @@ class Table(BaseWidgetTTK):
             case "elements":
                 self.overwrite_table(new_val)
             case _:
-                return False
+                return super()._update_special_key(key, new_val)
 
         return True
 
@@ -447,6 +430,7 @@ class Table(BaseWidgetTTK):
         # If the font changed, apply them to self._tk_kwargs
         if self.has_flag(ElementFlag.UPDATE_FONT):
             self._update_font()
+            self.remove_flags(ElementFlag.UPDATE_FONT)
 
         super()._apply_update() # Actually apply the update
 
@@ -487,7 +471,7 @@ class Table(BaseWidgetTTK):
         ).measure("X_") // 2
         if self._col_width_requested and self._font_size_multiplier != self._font_size_multiplier_applied:
             self._font_size_multiplier_applied = self._font_size_multiplier
-            self.update(column_width=self._col_width_requested)
+            self._update_initial(column_width=self._col_width_requested)
 
         # And now for the headings
         font_options = [
@@ -517,7 +501,7 @@ class Table(BaseWidgetTTK):
 
         return self._element_dict[str(hash(self._elements[temp]))]
 
-    def set_value(self,val:any):
+    def set_value(self,val:Any):
         print("Warning!","It is not possible to set Values of sg.Table (yet)!")
         print("     use .index to set the selected item")
 
@@ -702,8 +686,53 @@ class Table(BaseWidgetTTK):
 
         return row
 
+    #def throw_event(self, key: Any = None, value: Any= None, function: Callable= None, function_args: tuple = tuple(), function_kwargs: dict = None):
+    def _insert_multiple_thread_method(self,rows:Iterable[Iterable[Any]], index: int | Literal["end"], chunksize: int = 1000, delay: float = 0.1):
+        """
+        insert_multiple divided into chunks that get added one after another with a set delay
+        :param rows: Table-rows to add
+        :param index: Where to insert. "end" to append
+        :param chunksize: How many are added at once
+        :param delay: Time between additions
+        :return:
+        """
+        with self._thread_lock: # So the threads don't intercept each other
+            rows = iter(rows)
+
+            while True:
+                time.sleep(delay)
+                next_elems = list(islice(rows, chunksize))
+
+                if not next_elems:
+                    break
+
+                self.window.throw_event(function= self.insert_multiple, function_args= (next_elems, index, ))
+
+                if index != "end":
+                    index += chunksize
+
+    thread_running: bool    # True, if the insert-thread is running
+    @property
+    def thread_running(self) -> bool:
+        return self._thread_lock.locked()
+
+    _thread_lock: threading.Lock = None
     @BaseElement._run_after_window_creation
-    def insert_multiple(self,rows:Iterable[Iterable[Any]], index: int) -> tuple[TableRow, ...]:
+    def insert_multiple_threaded(self,rows:Iterable[Iterable[Any]], index: int | Literal["end"] = "end", chunksize: int = 1000, delay: float = 0.5) -> Self:
+        """
+        insert_multiple divided into chunks that get added one after another with a set delay
+        :param rows: Table-rows to add
+        :param index: Where to insert. "end" to append
+        :param chunksize: How many are added at once
+        :param delay: Time between additions
+        :return:
+        """
+        # Start the thread
+        threading.Thread(target= self._insert_multiple_thread_method, daemon= True, args=(rows, index, chunksize, delay)).start()
+        return self
+
+    @BaseElement._run_after_window_creation
+    def insert_multiple(self,rows:Iterable[Iterable[Any]], index: int | Literal["end"]) -> tuple[TableRow, ...]:
         """
         Insert multiple rows at once.
         They will be ordered like the list you pass, starting at the index you pass
@@ -711,6 +740,9 @@ class Table(BaseWidgetTTK):
         :param index:
         :return: Tuple of all the rows you added
         """
+        if index == "end":
+            return self.extend(rows)
+
         r = []
         for row in rows:
             r.append(self.insert(row,index))
@@ -732,7 +764,7 @@ class Table(BaseWidgetTTK):
         self._elements = list()
         self._element_dict = dict()
 
-    def overwrite_table(self, new_table: Iterable[Iterable[Any]]):
+    def overwrite_table(self, new_table: Iterable[Iterable[Any]]) -> Self:
         """
         Clear the whole table and replace its elements with a new table.
         :param new_table:
@@ -742,6 +774,20 @@ class Table(BaseWidgetTTK):
         self.clear_whole_table()
         # self._default_elements = list(self.insert_multiple(new_table, 0))
         self.insert_multiple(new_table, 0)
+        return self
+
+    def overwrite_table_threaded(self,rows:Iterable[Iterable[Any]], chunksize: int = 1000, delay: float = 0.3) -> Self:
+        """
+        Overwrite the whole table, but in small chunks to improve the performance
+        :param rows: Table-rows to add
+        :param chunksize: How many are added at once
+        :param delay: Time between additions
+        :return:
+        """
+        self.reset_filter()
+        self.clear_whole_table()
+        self.insert_multiple_threaded(rows, index= 0, chunksize= chunksize, delay= delay)
+        return self
 
     def append(self,row: Iterable[Any]) -> TableRow:
         """
@@ -761,6 +807,17 @@ class Table(BaseWidgetTTK):
         self.tk_widget.insert("",index = "end",values=row,iid=hash(row))
         return row
 
+    def extend_threaded(self,rows:Iterable[Iterable[Any]], chunksize: int = 1000, delay: float = 0.3) -> Self:
+        """
+        extend divided into chunks that get added one after another with a set delay
+        :param rows: Table-rows to add
+        :param chunksize: How many are added at once
+        :param delay: Time between additions
+        :return:
+        """
+        self.insert_multiple_threaded(rows, index= "end", chunksize= chunksize, delay= delay)
+        return self
+
     def extend(self,rows: Iterable[Iterable[Any]]) -> tuple[TableRow,...]:
         """
         Append multiple rows at once (like extend)
@@ -839,6 +896,7 @@ class Table(BaseWidgetTTK):
         self.move(index1, index2)
         return self
 
+    @BaseElement._run_after_window_creation
     def see(self, index: int = 0) -> Self:
         """
         Scroll through the list to see a certain index.
@@ -848,6 +906,20 @@ class Table(BaseWidgetTTK):
         self.tk_widget.see(str(hash(self._elements[index])))
         return self
 
+    @BaseElement._run_after_window_creation
+    def see_selection(self) -> Self:
+        """
+        Scroll the table so the current selection is visible.
+        If nothing is selected, this function does nothing.
+        :return:
+        """
+        if self.all_indexes:
+            self.see(self.all_indexes[0])
+        elif self.index is not None:
+            self.see(self.index)
+
+        return self
+
     @property
     def filter_mode(self) -> bool:
         """
